﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                Illusion                                  |
//|                              Version 1.00                                |
//|                                                                          |
//|      OpenGL project using the GImage flat API to create on the fly       |
//|      multiple textures with variable opacity from 32-bit PNG files.      |
//|                                                                          |
//|          The GDImage.dll unregistered version has no limitation,         |
//|             and all features of the API are fully functional.            |
//|                                                                          |
//|       Buying a commercial license removes the copyright nag screen.      |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-23-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-25-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;
//using System.Runtime;
//using System.IO;
//using Microsoft.Win32;

using Win32;
using GDImage;
using OpenGL;

namespace Illusion
{
    public partial class MAIN_Form : Form
    {
        const int ID_CTRL  = 100;                      // GDImage Zoom control
        const int TIMER_MAX = 1;                       // To perform smooth animation

        static GI.ZGLTEXTURE[] mt = new GI.ZGLTEXTURE[2];

        static public IntPtr hFORM_Main = IntPtr.Zero; // Default to Zero
        static public IntPtr hGLcontrol = IntPtr.Zero; // Default to Zero

        static GI.ZGLFONT UseFont = new GI.ZGLFONT();
        static bool Animate = false;

        static public float xrot;
        static public float yrot;
        static public float zrot;

        public MAIN_Form()
        {
            InitializeComponent();

            // Create GDImage control
            if (GI.RegisterGLImageClass())
            {
                // Create the OpenGL control
                hGLcontrol = GI.CreateControl(GL_Template, ID_CTRL);
                if (hGLcontrol != IntPtr.Zero)
                {
                    // This is our background image texture
                    mt[0].FullName = Api.ExePath() + "ilaero.png"; mt[0].ID = 10;
                    // This is our default moving sprite texture
                    mt[1].FullName = Api.ExePath() + "ilvista.jpg"; mt[1].ID = 20;

                    // Assign textures to mt[]
                    SetGenTextures(2);

                    // I am using a temporary ZGLTEXTURE structure 
                    // to protect mt[] from being erased during 
                    // call to SetMutipleGLTextureFrom32Bit.
                    GI.ZGLTEXTURE[] temp = new GI.ZGLTEXTURE[mt.Length];
                    for (int k = 0; k < mt.Length; k++) { temp[k] = mt[k]; }
                    if (GI.CreateMutiple32BitGLtextures(ref temp, mt.Length) == 0)
                    {
                        GL.glEnable(GL.GL_TEXTURE_2D);
                        // Clear backgorund to ARGB(255,255,255,255)
                        GL.glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
                        GL.glClearDepth(1.0);
                        GL.glDepthFunc(GL.GL_LESS);

                        GL.glShadeModel(GL.GL_SMOOTH);
		                GL.glMatrixMode(GL.GL_PROJECTION);
		
		                GL.glEnable(GL.GL_BLEND);
		                GL.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);

                        // When using the GDImage file selector
                        // start looking for 32-bit PNG (using variable opacity)   
                        // into the EXE folder.
                        GI.LoadSavePath(Api.ExePath(), true);
                    }
                }
            }

            hFORM_Main = this.Handle;
            StartAnimation();

        }

        private void SetGenTextures(int n)
        {
            int[] textures = new int[n];
            GL.glGenTextures(n, textures);
            for (int k = 0; k < mt.Length; k++) { mt[k].Texture = textures[k]; }
        }

        private void StartAnimation()
        {
            // Use API WM_TIMER to avoid aggressive garbage collection 
            // to occur before Main ends.
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.SetTimer(hFORM_Main, T + 1, 0, null);
            }
        }

        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {               
                case Api.WM_TIMER:
                    RenderOpenGL(hGLcontrol);
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_DESTROY:
                    DisposeGL(); // Free up OpenGL resources
                    break;
            }
            base.WndProc(ref m);
        }

        // Free up resources
        private void DisposeGL()
        {
            // Killtimer(s)
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.KillTimer(hFORM_Main, T + 1);
            }
            // Delete OpenGL font.
            GI.DeleteGLFont(ref UseFont);
        }

        private void RenderOpenGL(IntPtr hContainer)
        {
            if (OnOff.Checked)
            { GL.glEnable(GL.GL_DEPTH_TEST); }
            else
            { GL.glDisable(GL.GL_DEPTH_TEST); }

            GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
            GL.glLoadIdentity();
            // Draw the transparent background texture ------------------------
            GL.glBindTexture(GL.GL_TEXTURE_2D, mt[1].Texture);
            GL.glBegin(GL.GL_QUADS);
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f(-25.0f,-25.0f,-60.0f); // Bottom Left of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f( 25.0f,-25.0f,-60.0f); // Bottom Right of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f( 25.0f, 25.0f,-60.0f); // Top Right of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f(-25.0f, 25.0f,-60.0f); // Top Left of The Texture AND Quad
            GL.glEnd();
            // ----------------------------------------------------------------

            // Draw the transparent moving texture
            GL.glBindTexture(GL.GL_TEXTURE_2D, mt[0].Texture);

            GL.glTranslatef(0.0f, 0.0f, -5.0f);

            GL.glRotatef(xrot, 1.0f, 0.0f, 0.0f); // Rotate on The X Axis
            GL.glRotatef(yrot, 0.0f, 1.0f, 0.0f); // Rotate on The Y Axis
            GL.glRotatef(zrot, 0.0f, 0.0f, 1.0f); // Rotate on The Z Axis

            GL.glBegin(GL.GL_QUADS);

            // Back Face
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f(-1.0f,-1.0f,-1.0f); // Bottom Right of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f(-1.0f, 1.0f,-1.0f); // Top Right of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f( 1.0f, 1.0f,-1.0f); // Top Left of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f( 1.0f,-1.0f,-1.0f); // Bottom Left of The Texture AND Quad

            // Bottom Face
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f(-1.0f,-1.0f,-1.0f); // Top Right of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f( 1.0f,-1.0f,-1.0f); // Top Left of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f( 1.0f,-1.0f, 1.0f); // Bottom Left of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f(-1.0f,-1.0f, 1.0f); // Bottom Right of The Texture AND Quad

            // Right face
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f( 1.0f,-1.0f,-1.0f); // Bottom Right of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f( 1.0f, 1.0f,-1.0f); // Top Right of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f( 1.0f, 1.0f, 1.0f); // Top Left of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f( 1.0f,-1.0f, 1.0f); // Bottom Left of The Texture AND Quad

            // Left Face
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f(-1.0f,-1.0f,-1.0f); // Bottom Left of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f(-1.0f,-1.0f, 1.0f); // Bottom Right of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f(-1.0f, 1.0f, 1.0f); // Top Right of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f(-1.0f, 1.0f,-1.0f); // Top Left of The Texture AND Quad

            // Top Face
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f(-1.0f, 1.0f,-1.0f); // Top Left of The Texture AND Quad
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f(-1.0f, 1.0f, 1.0f); // Bottom Left of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f( 1.0f, 1.0f, 1.0f); // Bottom Right of The Texture AND Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f( 1.0f, 1.0f,-1.0f); // Top Right of The Texture AND Quad

            // Front Face
            GL.glTexCoord2f(0.0f, 0.0f); GL.glVertex3f(-1.0f,-1.0f, 1.0f); // Bottom Left Of The Texture And Quad
            GL.glTexCoord2f(1.0f, 0.0f); GL.glVertex3f( 1.0f,-1.0f, 1.0f); // Bottom Right Of The Texture And Quad
            GL.glTexCoord2f(1.0f, 1.0f); GL.glVertex3f( 1.0f, 1.0f, 1.0f); // Top Right Of The Texture And Quad
            GL.glTexCoord2f(0.0f, 1.0f); GL.glVertex3f(-1.0f, 1.0f, 1.0f); // Top Left Of The Texture And Quad

            GL.glEnd();

            // Draw the scene
            GI.UpdateGLWindow(hContainer);
            if (Animate)
            {
                xrot += 0.3f; // X Axis Rotation
                yrot += 0.2f; // Y Axis Rotation
                zrot += 0.4f; // Z Axis Rotation
            }

        }

        // ****************************************
        private void FORM_Main_Close(object sender, EventArgs e)
        {
        }

        //***********************************************************
        // We use the .NET template to resize the GDImage control
        private void FORM_Main_Resize(object sender, EventArgs e)
        {
            // Use the template to resize the GDImage OpenGL control
            Api.MoveWindow(hGLcontrol, GL_Template.Location.X, GL_Template.Location.Y, GL_Template.Width, GL_Template.Height, true);
            GI.ResizeGLWindow(hGLcontrol);
            RenderOpenGL(hGLcontrol);
        }

        private void BTN_Start_Click(object sender, EventArgs e)
        {
            Animate = true;
        }

        private void BTN_Select_Click(object sender, EventArgs e)
        {
            GI.LoadSavePath(Api.ExePath(), true);
            string sFileName = GI.LoadDialog(hFORM_Main);
            if (sFileName != null)
            {
                // Delete existing texture
                GL.glDeleteTextures(1, ref mt[0].Texture);
                mt[0].FullName = sFileName;
                // Load the new one

                // I am using a temporary ZGLTEXTURE structure 
                // to protect mt[] from being erased during 
                // call to SetMutipleGLTextureFrom32Bit.
                GI.ZGLTEXTURE[] temp = new GI.ZGLTEXTURE[mt.Length];
                for (int k = 0; k < mt.Length; k++) { temp[k] = mt[k]; }
                if (GI.CreateMutiple32BitGLtextures(ref temp, mt.Length) == 0)
                {
                    RenderOpenGL(hGLcontrol);
                }
            }
        }

        private void BTN_Stop_Click(object sender, EventArgs e)
        {
            Animate = false;
        }
    }
}