﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                              WIN32 class                                 |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-17-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-17-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Collections.Generic;
using System.Text;

using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Win32
{
    class Api
    {
        public const int SRCCOPY = 0x00CC0020;
        public const int MAX_PATH = 260;

        public const uint WS_EX_STATICEDGE = 0x00020000;
        public const uint WS_CHILD = 0x40000000;
        public const uint WS_VISIBLE = 0x10000000;
        public const uint WS_HSCROLL = 0x00100000;
        public const uint WS_VSCROLL = 0x00200000;

        public const int SB_HORZ = 0;
        public const int SB_VERT = 1;
        public const int SB_BOTH = 3;

        public const int WM_LBUTTONDOWN = 513;
        public const int WM_RBUTTONDOWN = 516;
        public const int WM_KEYDOWN = 256;
        public const int WM_MOUSEMOVE = 512;
        public const int WM_TIMER = 0x113;
        public const int WM_DESTROY = 2;
        public const int WM_ERASEBKGND = 0x14;
        public const int WM_PAINT = 0xF;
        public const int WM_COMMAND = 0x111;

        public const int VK_HOME = 36;
        public const int VK_END = 35;
        public const int VK_PRIOR = 33;
        public const int VK_NEXT = 34;
        public const int VK_LEFT = 37;
        public const int VK_UP = 38;
        public const int VK_RIGHT = 39;
        public const int VK_DOWN = 40;

        public const int VK_NUMPAD1 = 97;
        public const int VK_NUMPAD2 = 98;
        public const int VK_NUMPAD3 = 99;
        public const int VK_NUMPAD4 = 100;
        public const int VK_NUMPAD5 = 101;
        public const int VK_NUMPAD6 = 102;
        public const int VK_NUMPAD7 = 103;
        public const int VK_NUMPAD8 = 104;
        public const int VK_NUMPAD9 = 105;
        public const int VK_PGUP = 0x21;
        public const int VK_PGDN = 0x22;

        public const int FW_BOLD = 700;

        public const int BM_GETCHECK = 0xF0;
        public const int GWL_ID = -12;

        private const string USER32 = @"USER32.dll";
        private const string GDI32 = @"gdi32.dll";
        private const string KERNEL32 = @"KERNEL32.dll";

        [StructLayout(LayoutKind.Sequential)]
        public struct RECT { public int left, top, right, bottom;}

        public static int RGB(byte R, byte G, byte B) { return ((B * 256) + G) * 256 + R; }
        public static int ARGB(byte A, byte R, byte G, byte B) { return (((A * 256) + R) * 256 + G) * 256 + B; }

        public static byte RGBRed(int colrRGB) { return (byte)(colrRGB & 0x000000FF); }
        public static byte RGBGreen(int colrRGB) { return (byte)((colrRGB >> 8) & 0x000000FF); }
        public static byte RGBBlue(int colrRGB) { return (byte)((colrRGB >> 16) & 0x000000FF); }
        public static int LoWrd(uint value) { return (int)(value & 0xFFFF); }
        public static int HiWrd(uint value) { return (int)(value >> 16); }

        public static string ExePath()
        {
            return Application.StartupPath.TrimEnd('\\') + @"\";
        }

        public delegate void TimerProc(IntPtr hWnd, uint nMsg, int nIDEvent, int dwTime);
        [DllImport(USER32)]
        public static extern int SetTimer(IntPtr hWnd, int nIDEvent, int uElapse, TimerProc callback);

        [DllImport(USER32)]
        public static extern bool KillTimer(IntPtr hWnd, int nIDEvent);

        [DllImport(USER32)]
        public static extern bool GetWindowRect(IntPtr hWnd, ref RECT rw);

        [DllImport(USER32)]
        public static extern bool GetClientRect(IntPtr hWnd, ref RECT rc);

        [DllImport(GDI32)]  // Win32 encapsulation
        public static extern bool DeleteObject(IntPtr hObject);

        [DllImport(USER32)] // Win32 encapsulation
        public static extern IntPtr GetDesktopWindow();

        [DllImport(USER32)] // Win32 encapsulation
        public static extern int GetWindowDC(IntPtr hWnd);

        [DllImport(USER32)] // Win32 encapsulation
        public static extern int ReleaseDC(IntPtr hWnd, int DC);

        [DllImport(GDI32)]  // Win32 encapsulation
        public static extern int BitBlt(
        int hDCDest, int xDest, int yDest, int Width, int Heifght,
        int hDCSrce, int xSrce, int ySrce, int CopyMode
        );

        [DllImport(USER32)] // Win32 encapsulation
        public static extern IntPtr GetDlgItem(IntPtr hCtrl, int DlgItem);

        [DllImport(USER32)] // Win32 encapsulation
        public static extern void MoveWindow(IntPtr hCtrl, int X, int Y, int W, int H, bool Flag);

        [DllImport(USER32, EntryPoint = "GetWindowLongA")]
        public static extern int GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport(USER32, EntryPoint = "SetWindowLongA")]
        public static extern int SetWindowLong(IntPtr hWnd, int nIndex, int nNewLong);

        [DllImport(USER32, EntryPoint = "SendMessageA")]
        public static extern int SendMessage(IntPtr hWnd, uint dwMsg, uint wParam, int lParam);

        [DllImport(USER32, EntryPoint = "SetWindowTextA")]
        public static extern int SetWindowText(IntPtr hWnd, string lpString);

        [DllImport(USER32)]
        public static extern int GetDlgCtrlID(IntPtr hWnd);

        [DllImport(USER32)]
        public static extern IntPtr SetFocus(IntPtr hWnd);

        [DllImport(USER32, EntryPoint = "CreateWindowExA")]
        public static extern IntPtr CreateWindowEx(
            uint ExStyle,
            string ClassName,
            string Name,
            uint Style,
            int x,
            int y,
            int w,
            int h,
            IntPtr parent,
            int Menu,
            IntPtr Instance,
            int lpParam);

        [DllImport(KERNEL32, EntryPoint = "GetModuleHandleA")]
        public static extern IntPtr GetModuleHandle(string lpModuleName);

        [DllImport(USER32)]
        public static extern int ShowScrollBar(IntPtr hWnd, int iBar, int bShow);


    }
}
